#ifndef __DFBITERM_H__
#define __DFBITERM_H__

#include <wchar.h>
#include <wctype.h>
#include <signal.h>
#include <termios.h>		/* for termios structure */
#include <signal.h>		/* for signal() */
#include <sys/kd.h>		/* for KDSETMODE/KD_GRAPHICS/KD_TEXT */
#include <linux/vt.h>		/* for VT_* definitions */
#include <sys/ioctl.h>		/* for ioctl */
#include <directfb.h>
#include <pthread.h>

/* for VTcore access */
#include <iterm/core.h>		/* for VTCorem */
#include "VTScreenView.h"	/* for VTCore/VTScreenView */

#ifndef	FALSE
#  define FALSE 0
#endif
#ifndef	TRUE
#  define TRUE 1
#endif

#ifndef	min
#  define min(a,b) (((a)<(b))?(a):(b))
#endif
#ifndef	max
#  define max(a,b) (((a)>(b))?(a):(b))
#endif

#define DFBITERM_VERSION "0.1"

#define DefaultFont	"/usr/share/fonts/misc/8x16.pcf.gz"

enum ITERM_RETCODE
{
  ITERM_REPLACEFONT = 1,
};

enum ITERM_ERRCODE
{
  SUCCESS = 0,
  FONT_ERROR_EXIT = 1,
  TERM_ERROR_EXIT,
  FB_ERROR_EXIT,
  VT_ERROR_EXIT,
  INPUT_ERROR_EXIT,
};

typedef struct _Iterm
{
	pthread_t update_thread;
	VTCore *vtcore_ptr;		/* VTCore pointer */
	VTScreenView *view;		/* VT screen view pointer */
	TerminalIO *tio;		/* terminal IO data pointer */
	IDirectFB *dfb;			/* DirectFB */
	IDirectFBSurface *surface;
	IDirectFBFont *font;
	IDirectFBPalette *palette;
	IDirectFBWindow *window;
	IDirectFBDisplayLayer *layer;
	IDirectFBEventBuffer *buffer;
	struct termios ntio;		/* for backup */
	int fd;				/* fd for writing */
	int indexed;
	int screen_width;
	int screen_height;
	int cell_width;		/* cell width per character */
	int cell_height;		/* cell height per character */
	int cols;			/* total columns (xres / cell_width) */
	int rows;			/* total rows (yres / cell_height) */
	int bold;			/* bold flag */
	int blink;			/* blink flag */
	int underline;		/* underline flag */
	int background;
	int foreground;
	int inverse;
	char charset;			/* char area for distinguishing UK or special graph */
	int lock;			/* lock flag for the display control 
				   at the time of the change of a virtual terminal */
}
Iterm;


/* function prototypes */
int InitDFb ();
int InitTerm ();
int InitVt ();
int InitInput (void);


void clean_up(int);
void dfb_end(void);
void term_handle_key(DFBWindowEvent *);
void term_handle_button(DFBWindowEvent *);
void setup_vt_handle (void);

extern void VTCore_set_direction ();
extern Iterm *pIterm;

#endif /* __DFBITERM_H__ */
