#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "dfbiterm.h"
#include <directfb.h>

/* remapping table for function keys 5-12 */
static const unsigned char f5_f12_remap[] = {15,17,18,19,20,21,23,24};

void
term_handle_key(DFBWindowEvent *evt)
{
	if (evt->type == DWET_KEYUP)
		return;

	if (evt->modifiers  == DIMM_CONTROL && evt->key_symbol >= DIKS_SMALL_A && evt->key_symbol <= DIKS_SMALL_Z) {
		char c = evt->key_symbol - DIKS_SMALL_A + 1;

		write(pIterm->fd, &c, 1);
	}
	else if ((evt->key_symbol >   9 && evt->key_symbol < 127) || (evt->key_symbol > 127 && evt->key_symbol < 256)) {
		char c = evt->key_symbol & 0xff;

		if (evt->modifiers & DIMM_CONTROL) {
			switch (evt->key_symbol) {
			case ' ':
				write(pIterm->fd, "\000", 1);
				break;
			case '3':
			case '[':
				write(pIterm->fd, "\033", 1);
				break;
			case '4':
			case '\\':
				write(pIterm->fd, "\034", 1);
				break;
			case '5':
			case ']':
				write(pIterm->fd, "\035", 1);
				break;
			case '6':
				write(pIterm->fd, "\036", 1);
				break;
			case '7':
			case '-':
				write(pIterm->fd, "\037", 1);
				break;
			default:
				write(pIterm->fd, &c, 1);
				break;
			}
		}
		else
			write(pIterm->fd, &c, 1);
	}
	else {
		switch (evt->key_symbol) {
		case DIKS_BACKSPACE:
			write(pIterm->fd, "\177", 1);
			break;
		case DIKS_TAB:
			if (evt->modifiers & DIMM_SHIFT)   /* back tab */
				write(pIterm->fd, "\033[Z", 3);
			else
				write(pIterm->fd, "\t", 1);
			break;
		case DIKS_DELETE:
			write(pIterm->fd, "\033[3~", 4);
			break;
		case DIKS_INSERT:
			write(pIterm->fd, "\033[2~", 4);
			break;
		case DIKS_CURSOR_LEFT:
			write(pIterm->fd, "\033OD", 3);
			break;
		case DIKS_CURSOR_RIGHT:
			write(pIterm->fd, "\033OC", 3);
			break;
		case DIKS_CURSOR_UP:
			if (evt->modifiers & DIMM_SHIFT) {
				VTCore_scroll_up (pIterm->vtcore_ptr, 1);
				return;
			} else {
				write(pIterm->fd, "\033OA", 3);
			}
			break;
		case DIKS_CURSOR_DOWN:
			if (evt->modifiers & DIMM_SHIFT) {
				VTCore_scroll_down(pIterm->vtcore_ptr, 1);
				return;
			} else {
				write(pIterm->fd, "\033OB", 3);
			}
			break;
		case DIKS_HOME:
			write(pIterm->fd, "\033OH", 3);
			break;
		case DIKS_END:
			write(pIterm->fd, "\033OF", 3);
			break;
		case DIKS_PAGE_UP:
			if (evt->modifiers & DIMM_SHIFT) {
				VTCore_scroll_up(pIterm->vtcore_ptr, pIterm->rows / 2);
				return;
			} else
				write(pIterm->fd, "\033[5~", 4);
			break;
		case DIKS_PAGE_DOWN:
			if (evt->modifiers & DIMM_SHIFT) {
				VTCore_scroll_down(pIterm->vtcore_ptr, pIterm->rows / 2);
				return;
			} else
				write(pIterm->fd, "\033[6~", 4);
			break;
		case DIKS_F1:
			write(pIterm->fd, "\033OP", 3);
			break;
		case DIKS_F2:
			write(pIterm->fd, "\033OQ", 3);
			break;
		case DIKS_F3:
			write(pIterm->fd, "\033OR", 3);
			break;
		case DIKS_F4:
			write(pIterm->fd, "\033OS", 3);
			break;
		case DIKS_F5 ... DIKS_F12:
			{
				char buf[6];

				sprintf (buf, "\033[%d~", f5_f12_remap[evt->key_symbol - DIKS_F5]);
				write(pIterm->fd, buf, strlen(buf));
			}
			break;
		default:
			return;
		}
	}
}
