#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>

#include "dfbiterm.h"		/* for iterm_struct */

static void
init_tty(void)
{
	struct termios t;
	write(STDOUT_FILENO, "\x1b=",2); /* to switch keypad application modes */

	/* save current parameters */
	tcgetattr (STDIN_FILENO, &(pIterm->ntio));

	/* set new keyboard control parameters for new terminal */
	t = pIterm->ntio;
	t.c_line = 0;
	t.c_lflag &= ~(ECHO | ISIG | ICANON | XCASE);
	t.c_iflag = 0;
	t.c_cflag |= CS8;
	t.c_oflag &= ~OPOST;
	t.c_cc[VMIN] = 1;
	t.c_cc[VTIME] = 0;
	tcsetattr (0, TCSAFLUSH, &t);
}

void
clean_up(int sig)
{
	/* set new keyboard control parameters for new terminal */
	write(STDOUT_FILENO, "\x1b>",2); /* to switch keypad numeric modes */
	tcsetattr (0, TCSAFLUSH, &(pIterm->ntio));
	dfb_end();
	exit (0);
}

/* Initialize tty */
int
InitTerm(void)
{
	char *tty_name;
	int display_mode;
  
	if (!isatty(STDIN_FILENO)) {
		fprintf (stderr, "This is not a tty.\r\n");
		return -1;
	}
  
	tty_name = ttyname (STDIN_FILENO);
	if (!tty_name) {
		fprintf (stderr, "Cannot get current tty name.\r\n");
		return -1;
	}

	if ((strncmp ("/dev/tty", tty_name, 8) != 0) && (strncmp("/dev/vc/", tty_name, 8) != 0)) {
		fprintf (stderr,"This is not real virtual terminal %s.\r\n",tty_name);
		return -1;
	}

	/* examining if /dev/tty[0-9]+. "/dev/tty" may be part of ttyS? or ttyp? */
	if(strlen(tty_name) < 8 || tty_name[8] < '0' || '9' < tty_name[8]) {
		fprintf (stderr,"This is not real virtual terminal %s.\r\n", tty_name);
		return -1;
	} 

	pIterm->lock = FALSE;
/*	init_tty(); */
	return 0;
}
